#!/usr/bin/python
# encoding: utf-8
import time
from ftplib import FTP
from ftplib import FTP_TLS
import sys
import hashlib
import os
import ssl


def get_absolute_path(path):
    abs_path = os.path.abspath(path)
    if not os.path.exists(abs_path):
        raise FileNotFoundError(f"路径不存在: {abs_path}")
    return abs_path


def generate_remote_path(base_path, file_name):
    if not base_path.endswith('/'):
        base_path += '/'
    return base_path + file_name


class MyFTP(FTP):
    def __init__(self):
        print('make a object')

    def ConnectFTP(self, remote_ip="", remote_port=21, login_name="", login_password=""):
        ftp = FTP_TLS()
        ftp.ssl_version = ssl.PROTOCOL_TLSv1_2

        try:
            ftp.connect(remote_ip, remote_port, 600)
            print('Connect Success')
        except Exception as e:
            print(f"Connect Failed - {e}", file=sys.stderr)
            return (0, 'Connect Failed')
        else:
            try:
                ftp.login(login_name, login_password)
                print('Login Success')
            except Exception as e:
                print(f"Login Failed - {e}", file=sys.stderr)
                time.sleep(10)
                return (0, 'Login Failed')
            else:
                return (1, ftp)

    def upload(self, remoteHost, remotePort, login_name, login_password, remoteBasePath, localPath, upload_filename=None):
        localPath = get_absolute_path(localPath)
        localFileName = os.path.basename(localPath)

        remoteFilename = upload_filename if upload_filename else localFileName

        res = self.ConnectFTP(remote_ip=remoteHost, remote_port=remotePort, login_name=login_name,
                              login_password=login_password)

        if res[0] != 1:
            print(res[1])
            sys.exit()
        ftp = res[1]

        try:
            ftp.cwd(remoteBasePath)
        except Exception:
            # Dynamically create directories
            parts = remoteBasePath.strip('/').split('/')
            current_path = ''
            for part in parts:
                current_path = f"{current_path}/{part}" if current_path else part
                try:
                    ftp.cwd(current_path)
                except Exception:
                    ftp.mkd(current_path)
                    ftp.cwd(current_path)

        # Check if the remote file exists
        try:
            rsize = ftp.size(remoteFilename)
            print(f"File {remoteFilename} already exists on the server.")
            delete = input("Do you want to delete it and re-upload? (y/n): ").strip().lower()
            if delete == 'y':
                ftp.delete(remoteFilename)
                print(f"Deleted {remoteFilename} from the server.")
                rsize = 0  # Start fresh upload
            else:
                print("Aborting upload. Please delete the file manually if necessary.")
                ftp.quit()
                return
        except Exception:
            rsize = 0  # File does not exist, proceed with upload

        lsize = os.stat(localPath).st_size
        print(f"Local file size: {lsize} bytes, Remote file size: {rsize} bytes")

        uploaded = [rsize]  # Track uploaded bytes

        def progress_callback(data):
            uploaded[0] += len(data)  # Update uploaded bytes count
            percent = (uploaded[0] / lsize) * 100
            progress_bar = ('#' * int(percent // 2)).ljust(50)  # Progress bar: one '#' per 2%
            print(f"\r[{progress_bar}] {percent:.2f}% ({uploaded[0]}/{lsize} bytes)", end='')

        with open(localPath, 'rb') as localfile:
            if rsize > 0:
                localfile.seek(rsize)  # Resume upload from the last position
            ftp.storbinary(f"STOR {remoteFilename}", localfile, callback=progress_callback)

        print(f"\nUploaded {localPath} to {remoteFilename}")
        ftp.quit()

    def splitpath(self, remotepath):
        position = remotepath.rfind('/')
        return (remotepath[:position + 1], remotepath[position + 1:])

    def md5_calc(self, file):
        md5_value = hashlib.md5()
        with open(file, 'rb') as file_b:
            while True:
                data_flow = file_b.read(8096)
                if not data_flow:
                    break
                md5_value.update(data_flow)
        return md5_value.hexdigest()

    def get_filesize(self, filename):
        fsize = os.path.getsize(filename)
        return fsize


def main():
    localfile = input("Enter the file name to upload: ").strip()
    login_name = input("Enter FTP username: ").strip()
    login_password = input("Enter FTP password: ").strip()
    upload_filename = input("Enter the new name for the file on the server: ").strip()

    ftpobj = MyFTP()
    ftpobj.upload(
        remoteHost='165.73.249.37',
        remotePort=10021,
        login_name=login_name,
        login_password=login_password,
        remoteBasePath="/ftp-download/vps-clone/",
        upload_filename=upload_filename if upload_filename else None,
        localPath=localfile
    )


if __name__ == "__main__":
    try:
        main()
        input("Upload complete. Press Enter to exit...")
    except Exception as e:
        print(f"An error occurred: {e}", file=sys.stderr)
        input("Press Enter to exit...")
        sys.exit(1)

